@@ -33,9 +33,10 @@ import math
 
 catalog = i18nCatalog("cura")
 
-## Standard view for mesh models.
 
 class SolidView(View):
+    """Standard view for mesh models."""
+
     _show_xray_warning_preference = "view/show_xray_warning"
 
     def __init__(self):
@@ -63,17 +64,23 @@ class SolidView(View):
         self._old_layer_bindings = None
 
         self._next_xray_checking_time = time.time()
-        self._xray_checking_update_time = 1.0 # seconds
+        self._xray_checking_update_time = 30.0 # seconds
         self._xray_warning_cooldown = 60 * 10 # reshow Model error message every 10 minutes
         self._xray_warning_message = Message(
             catalog.i18nc("@info:status", "Your model is not manifold. The highlighted areas indicate either missing or extraneous surfaces."),
             lifetime = 60 * 5, # leave message for 5 minutes
             title = catalog.i18nc("@info:title", "Model errors"),
+            option_text = catalog.i18nc("@info:option_text", "Do not show this message again"),
+            option_state = False
         )
+        self._xray_warning_message.optionToggled.connect(self._onDontAskMeAgain)
         application.getPreferences().addPreference(self._show_xray_warning_preference, True)
 
         application.engineCreatedSignal.connect(self._onGlobalContainerChanged)
 
+    def _onDontAskMeAgain(self, checked: bool) -> None:
+        Application.getInstance().getPreferences().setValue(self._show_xray_warning_preference, not checked)
+
     def _onGlobalContainerChanged(self) -> None:
         if self._global_stack:
             try:
@@ -102,7 +109,9 @@ class SolidView(View):
             except IndexError:
                 pass
             else:
-                self._support_angle = support_angle_stack.getProperty("support_angle", "value")
+                angle = support_angle_stack.getProperty("support_angle", "value")
+                if angle is not None:
+                    self._support_angle = angle
 
     def _checkSetup(self):
         if not self._extruders_model:
@@ -114,6 +123,7 @@ class SolidView(View):
         if not self._enabled_shader:
             self._enabled_shader = OpenGL.getInstance().createShaderProgram(Resources.getPath(Resources.Shaders, "overhang.shader"))
             self._enabled_shader.setUniformValue("u_overhangColor", Color(*self._theme.getColor("model_overhang").getRgb()))
+            self._enabled_shader.setUniformValue("u_renderError", 0.0)
 
         if not self._disabled_shader:
             self._disabled_shader = OpenGL.getInstance().createShaderProgram(Resources.getPath(Resources.Shaders, "striped.shader"))
@@ -139,6 +149,7 @@ class SolidView(View):
                 self._composite_pass.setCompositeShader(self._old_composite_shader)
                 self._old_layer_bindings = None
                 self._old_composite_shader = None
+                self._enabled_shader.setUniformValue("u_renderError", 0.0)  # We don't want any error markers!.
                 self._xray_warning_message.hide()
         else:
             if not self._xray_shader:
@@ -149,13 +160,14 @@ class SolidView(View):
                 theme = Application.getInstance().getTheme()
                 self._xray_composite_shader.setUniformValue("u_background_color", Color(*theme.getColor("viewport_background").getRgb()))
                 self._xray_composite_shader.setUniformValue("u_outline_color", Color(*theme.getColor("model_selection_outline").getRgb()))
+                self._xray_composite_shader.setUniformValue("u_flat_error_color_mix", 0.)  # Don't show flat error color in solid-view.
 
             renderer = self.getRenderer()
             if not self._composite_pass or not 'xray' in self._composite_pass.getLayerBindings():
                 # Currently the RenderPass constructor requires a size > 0
                 # This should be fixed in RenderPass's constructor.
                 self._xray_pass = XRayPass.XRayPass(1, 1)
-
+                self._enabled_shader.setUniformValue("u_renderError", 1.0)  # We don't want any error markers!.
                 renderer.addRenderPass(self._xray_pass)
 
                 if not self._composite_pass:
@@ -176,77 +188,77 @@ class SolidView(View):
         if global_container_stack:
             if Application.getInstance().getPreferences().getValue("view/show_overhang"):
                 # Make sure the overhang angle is valid before passing it to the shader
-                if self._support_angle is not None and self._support_angle >= 0 and self._support_angle <= 90:
+                if self._support_angle >= 0 and self._support_angle <= 90:
                     self._enabled_shader.setUniformValue("u_overhangAngle", math.cos(math.radians(90 - self._support_angle)))
                 else:
                     self._enabled_shader.setUniformValue("u_overhangAngle", math.cos(math.radians(0))) #Overhang angle of 0 causes no area at all to be marked as overhang.
             else:
                 self._enabled_shader.setUniformValue("u_overhangAngle", math.cos(math.radians(0)))
-
+        disabled_batch = renderer.createRenderBatch(shader = self._disabled_shader)
+        normal_object_batch = renderer.createRenderBatch(shader = self._enabled_shader)
+        renderer.addRenderBatch(disabled_batch)
+        renderer.addRenderBatch(normal_object_batch)
         for node in DepthFirstIterator(scene.getRoot()):
-            if not node.render(renderer):
-                if node.getMeshData() and node.isVisible() and not node.callDecoration("getLayerData"):
-                    uniforms = {}
-                    shade_factor = 1.0
+            if node.render(renderer):
+                continue
 
-                    per_mesh_stack = node.callDecoration("getStack")
+            if node.getMeshData() and node.isVisible():
+                uniforms = {}
+                shade_factor = 1.0
 
-                    extruder_index = node.callDecoration("getActiveExtruderPosition")
-                    if extruder_index is None:
-                        extruder_index = "0"
-                    extruder_index = int(extruder_index)
+                per_mesh_stack = node.callDecoration("getStack")
 
-                    # Use the support extruder instead of the active extruder if this is a support_mesh
-                    if per_mesh_stack:
-                        if per_mesh_stack.getProperty("support_mesh", "value"):
-                            extruder_index = int(global_container_stack.getExtruderPositionValueWithDefault("support_extruder_nr"))
+                extruder_index = node.callDecoration("getActiveExtruderPosition")
+                if extruder_index is None:
+                    extruder_index = "0"
+                extruder_index = int(extruder_index)
 
-                    try:
-                        material_color = self._extruders_model.getItem(extruder_index)["color"]
-                    except KeyError:
-                        material_color = self._extruders_model.defaultColors[0]
+                try:
+                    material_color = self._extruders_model.getItem(extruder_index)["color"]
+                except KeyError:
+                    material_color = self._extruders_model.defaultColors[0]
 
-                    if extruder_index != ExtruderManager.getInstance().activeExtruderIndex:
-                        # Shade objects that are printed with the non-active extruder 25% darker
-                        shade_factor = 0.6
+                if extruder_index != ExtruderManager.getInstance().activeExtruderIndex:
+                    # Shade objects that are printed with the non-active extruder 25% darker
+                    shade_factor = 0.6
 
-                    try:
-                        # Colors are passed as rgb hex strings (eg "#ffffff"), and the shader needs
-                        # an rgba list of floats (eg [1.0, 1.0, 1.0, 1.0])
-                        uniforms["diffuse_color"] = [
-                            shade_factor * int(material_color[1:3], 16) / 255,
-                            shade_factor * int(material_color[3:5], 16) / 255,
-                            shade_factor * int(material_color[5:7], 16) / 255,
-                            1.0
-                        ]
+                try:
+                    # Colors are passed as rgb hex strings (eg "#ffffff"), and the shader needs
+                    # an rgba list of floats (eg [1.0, 1.0, 1.0, 1.0])
+                    uniforms["diffuse_color"] = [
+                        shade_factor * int(material_color[1:3], 16) / 255,
+                        shade_factor * int(material_color[3:5], 16) / 255,
+                        shade_factor * int(material_color[5:7], 16) / 255,
+                        1.0
+                    ]
 
-                        # Color the currently selected face-id. (Disable for now.)
-                        #face = Selection.getHoverFace()
-                        uniforms["hover_face"] = -1 #if not face or node != face[0] else face[1]
-                    except ValueError:
-                        pass
+                    # Color the currently selected face-id. (Disable for now.)
+                    #face = Selection.getHoverFace()
+                    uniforms["hover_face"] = -1 #if not face or node != face[0] else face[1]
+                except ValueError:
+                    pass
 
-                    if node.callDecoration("isNonPrintingMesh"):
-                        if per_mesh_stack and (per_mesh_stack.getProperty("infill_mesh", "value") or per_mesh_stack.getProperty("cutting_mesh", "value")):
-                            renderer.queueNode(node, shader = self._non_printing_shader, uniforms = uniforms, transparent = True)
-                        else:
-                            renderer.queueNode(node, shader = self._non_printing_shader, transparent = True)
-                    elif getattr(node, "_outside_buildarea", False):
-                        renderer.queueNode(node, shader = self._disabled_shader)
-                    elif per_mesh_stack and per_mesh_stack.getProperty("support_mesh", "value"):
-                        # Render support meshes with a vertical stripe that is darker
-                        shade_factor = 0.6
-                        uniforms["diffuse_color_2"] = [
-                            uniforms["diffuse_color"][0] * shade_factor,
-                            uniforms["diffuse_color"][1] * shade_factor,
-                            uniforms["diffuse_color"][2] * shade_factor,
-                            1.0
-                        ]
-                        renderer.queueNode(node, shader = self._support_mesh_shader, uniforms = uniforms)
+                if node.callDecoration("isNonPrintingMesh"):
+                    if per_mesh_stack and (node.callDecoration("isInfillMesh") or node.callDecoration("isCuttingMesh")):
+                        renderer.queueNode(node, shader = self._non_printing_shader, uniforms = uniforms, transparent = True)
                     else:
-                        renderer.queueNode(node, shader = self._enabled_shader, uniforms = uniforms)
-                if node.callDecoration("isGroup") and Selection.isSelected(node):
-                    renderer.queueNode(scene.getRoot(), mesh = node.getBoundingBoxMesh(), mode = RenderBatch.RenderMode.LineLoop)
+                        renderer.queueNode(node, shader = self._non_printing_shader, transparent = True)
+                elif getattr(node, "_outside_buildarea", False):
+                    disabled_batch.addItem(node.getWorldTransformation(copy = False), node.getMeshData(), normal_transformation = node.getCachedNormalMatrix())
+                elif per_mesh_stack and node.callDecoration("isSupportMesh"):
+                    # Render support meshes with a vertical stripe that is darker
+                    shade_factor = 0.6
+                    uniforms["diffuse_color_2"] = [
+                        uniforms["diffuse_color"][0] * shade_factor,
+                        uniforms["diffuse_color"][1] * shade_factor,
+                        uniforms["diffuse_color"][2] * shade_factor,
+                        1.0
+                    ]
+                    renderer.queueNode(node, shader = self._support_mesh_shader, uniforms = uniforms)
+                else:
+                    normal_object_batch.addItem(node.getWorldTransformation(copy=False), node.getMeshData(), uniforms=uniforms, normal_transformation = node.getCachedNormalMatrix())
+            if node.callDecoration("isGroup") and Selection.isSelected(node):
+                renderer.queueNode(scene.getRoot(), mesh = node.getBoundingBoxMesh(), mode = RenderBatch.RenderMode.LineLoop)
 
     def endRendering(self):
         # check whether the xray overlay is showing badness
@@ -267,13 +279,23 @@ class SolidView(View):
                 Class that ducktypes to be a Numpy ndarray.
                 """
                 def __init__(self, qimage):
-                    self.__array_interface__ = {
-                        "shape": (qimage.height(), qimage.width()),
-                        "typestr": "|u4", # Use 4 bytes per pixel rather than 3, since Numpy doesn't support 3.
-                        "data": (int(qimage.bits()), False),
-                        "strides": (qimage.bytesPerLine(), 3),  # This does the magic: For each line, skip the correct number of bytes. Bytes per pixel is always 3 due to QImage.Format.Format_RGB888.
-                        "version": 3
-                    }
+                    bits_pointer = qimage.bits()
+                    if bits_pointer is None:  # If this happens before there is a window.
+                        self.__array_interface__ = {
+                            "shape": (0, 0),
+                            "typestr": "|u4",
+                            "data": (0, False),
+                            "strides": (1, 3),
+                            "version": 3
+                        }
+                    else:
+                        self.__array_interface__ = {
+                            "shape": (qimage.height(), qimage.width()),
+                            "typestr": "|u4", # Use 4 bytes per pixel rather than 3, since Numpy doesn't support 3.
+                            "data": (int(bits_pointer), False),
+                            "strides": (qimage.bytesPerLine(), 3),  # This does the magic: For each line, skip the correct number of bytes. Bytes per pixel is always 3 due to QImage.Format.Format_RGB888.
+                            "version": 3
+                        }
             array = np.asarray(QImageArrayView(xray_img)).view(np.dtype({
                 "r": (np.uint8, 0, "red"),
                 "g": (np.uint8, 1, "green"),
