@@ -22,8 +22,10 @@ from cura.Scene.SliceableObjectDecorator import SliceableObjectDecorator  # Adde
 if TYPE_CHECKING:
     from UM.Scene.SceneNode import SceneNode
 
-##  Class that leverages Trimesh to import files.
+
 class TrimeshReader(MeshReader):
+    """Class that leverages Trimesh to import files."""
+
     def __init__(self) -> None:
         super().__init__()
 
@@ -79,11 +81,13 @@ class TrimeshReader(MeshReader):
             )
         )
 
-    ##  Reads a file using Trimesh.
-    #   \param file_name The file path. This is assumed to be one of the file
-    #   types that Trimesh can read. It will not be checked again.
-    #   \return A scene node that contains the file's contents.
     def _read(self, file_name: str) -> Union["SceneNode", List["SceneNode"]]:
+        """Reads a file using Trimesh.
+
+        :param file_name: The file path. This is assumed to be one of the file
+        types that Trimesh can read. It will not be checked again.
+        :return: A scene node that contains the file's contents.
+        """
         # CURA-6739
         # GLTF files are essentially JSON files. If you directly give a file name to trimesh.load(), it will
         # try to figure out the format, but for GLTF, it loads it as a binary file with flags "rb", and the json.load()
@@ -130,13 +134,14 @@ class TrimeshReader(MeshReader):
             node.setParent(group_node)
         return group_node
 
-    ##  Converts a Trimesh to Uranium's MeshData.
-    #   \param tri_node A Trimesh containing the contents of a file that was
-    #   just read.
-    #   \param file_name The full original filename used to watch for changes
-    #   \return Mesh data from the Trimesh in a way that Uranium can understand
-    #   it.
     def _toMeshData(self, tri_node: trimesh.base.Trimesh, file_name: str = "") -> MeshData:
+        """Converts a Trimesh to Uranium's MeshData.
+
+        :param tri_node: A Trimesh containing the contents of a file that was just read.
+        :param file_name: The full original filename used to watch for changes
+        :return: Mesh data from the Trimesh in a way that Uranium can understand it.
+        """
+
         tri_faces = tri_node.faces
         tri_vertices = tri_node.vertices
 
