@@ -7,9 +7,9 @@ from UM.Message import Message
 I18N_CATALOG = i18nCatalog("cura")
 
 
-## Message shown when uploading a print job to a cluster failed.
 class PrintJobUploadErrorMessage(Message):
-    
+    """Message shown when uploading a print job to a cluster failed."""
+
     def __init__(self, message: str = None) -> None:
         super().__init__(
             text = message or I18N_CATALOG.i18nc("@info:text", "Could not upload the data to the printer."),
