@@ -5,12 +5,11 @@ from typing import Dict, Any
 from ..BaseModel import BaseModel
 
 
-## Class representing the system status of a printer.
 class PrinterSystemStatus(BaseModel):
+    """Class representing the system status of a printer."""
 
     def __init__(self, guid: str, firmware: str, hostname: str, name: str, platform: str, variant: str,
-                 hardware: Dict[str, Any], **kwargs
-                 ) -> None:
+                 hardware: Dict[str, Any], **kwargs) -> None:
         self.guid = guid
         self.firmware = firmware
         self.hostname = hostname
