@@ -3,6 +3,7 @@ from unittest.mock import MagicMock, patch
 import pytest
 
 from cura.API import Account
+from cura.API.Account import SyncState
 from cura.OAuth2.Models import UserProfile
 
 
@@ -19,16 +20,24 @@ def test_login():
     account = Account(MagicMock())
     mocked_auth_service = MagicMock()
     account._authorization_service = mocked_auth_service
+    account.logout = MagicMock()
 
     account.login()
-    mocked_auth_service.startAuthorizationFlow.assert_called_once_with()
+    mocked_auth_service.startAuthorizationFlow.assert_called_once_with(False)
 
-    # Fake a sucesfull login
+    # Fake a successful login
     account._onLoginStateChanged(True)
 
     # Attempting to log in again shouldn't change anything.
     account.login()
-    mocked_auth_service.startAuthorizationFlow.assert_called_once_with()
+    mocked_auth_service.startAuthorizationFlow.assert_called_once_with(False)
+
+    # Attempting to log in with force_logout_before_login as True should call the logout before calling the
+    # startAuthorizationFlow(True).
+    account.login(force_logout_before_login=True)
+    account.logout.assert_called_once_with()
+    mocked_auth_service.startAuthorizationFlow.assert_called_with(True)
+    assert mocked_auth_service.startAuthorizationFlow.call_count == 2
 
 
 def test_initialize():
@@ -109,3 +118,55 @@ def test_userProfile(user_profile):
 
     mocked_auth_service.getUserProfile = MagicMock(return_value=None)
     assert account.userProfile is None
+
+
+def test_sync_success():
+    account = Account(MagicMock())
+
+    service1 = "test_service1"
+    service2 = "test_service2"
+
+    account.setSyncState(service1, SyncState.SYNCING)
+    assert account.syncState == SyncState.SYNCING
+
+    account.setSyncState(service2, SyncState.SYNCING)
+    assert account.syncState == SyncState.SYNCING
+
+    account.setSyncState(service1, SyncState.SUCCESS)
+    # service2 still syncing
+    assert account.syncState == SyncState.SYNCING
+
+    account.setSyncState(service2, SyncState.SUCCESS)
+    assert account.syncState == SyncState.SUCCESS
+
+
+def test_sync_update_action():
+    account = Account(MagicMock())
+
+    service1 = "test_service1"
+
+    mockUpdateCallback = MagicMock()
+
+    account.setSyncState(service1, SyncState.SYNCING)
+    assert account.syncState == SyncState.SYNCING
+
+    account.setUpdatePackagesAction(mockUpdateCallback)
+    account.onUpdatePackagesClicked()
+    mockUpdateCallback.assert_called_once_with()
+    account.setSyncState(service1, SyncState.SUCCESS)
+
+    account.sync()  # starting a new sync resets the update action to None
+
+    account.setSyncState(service1, SyncState.SYNCING)
+    assert account.syncState == SyncState.SYNCING
+
+    account.onUpdatePackagesClicked()  # Should not be connected to an action anymore
+    mockUpdateCallback.assert_called_once_with()  # No additional calls
+    assert account.updatePackagesEnabled is False
+    account.setSyncState(service1, SyncState.SUCCESS)
+
+
+
+
+
+
