@@ -1,3 +1,6 @@
+# Copyright (c) 2020 Ultimaker B.V.
+# Cura is released under the terms of the LGPLv3 or higher.
+
 from unittest.mock import MagicMock, patch
 import pytest
 
@@ -6,9 +9,6 @@ from UM.Math.Vector import Vector
 from cura.BuildVolume import BuildVolume, PRIME_CLEARANCE
 import numpy
 
-
-
-
 @pytest.fixture
 def build_volume() -> BuildVolume:
     mocked_application = MagicMock()
@@ -161,12 +161,10 @@ class TestUpdateRaftThickness:
             return properties.get(args[2])
 
     def createMockedStack(self):
-        mocked_global_stack = MagicMock(name="mocked_global_stack")
-        mocked_global_stack.getProperty = MagicMock(side_effect=self.getPropertySideEffect)
+        mocked_global_stack = MagicMock(name = "mocked_global_stack")
+        mocked_global_stack.getProperty = MagicMock(side_effect = self.getPropertySideEffect)
         extruder_stack = MagicMock()
 
-        mocked_global_stack.extruders = {"0": extruder_stack}
-
         return mocked_global_stack
 
     def test_simple(self, build_volume: BuildVolume):
@@ -254,7 +252,7 @@ class TestCalculateExtraZClearance:
             return properties.get(args[2])
 
     def test_noContainerStack(self, build_volume: BuildVolume):
-        assert build_volume._calculateExtraZClearance([]) is 0
+        assert build_volume._calculateExtraZClearance([]) == 0
 
     def test_withRetractionHop(self, build_volume: BuildVolume):
         mocked_global_stack = MagicMock(name="mocked_global_stack")
